#include "brain.h"
#include <stdlib.h>
#include <stdio.h>
#include <signal.h>
#include <time.h>

#include "v5.h"
#include "chrtab.h"

#define xdim V5_ANALYSOI
#define ydim V5_LENGTH

char dah[xdim*ydim] =
"........................................" //0
"..........X............................."
"..........X............................."
"...........X............................"
"........................................"
".......X................................" //5
"........................................"
".......X................................"
".......X................................"
".......X................................"
"........................................" //10
"........................................"
"........................................"
"........................................"
"........................................";
                                           //15

#define numout V5_NUMOUTPUT

double in[ydim][xdim];
double out[numout];

static void Noisify(int Draw)
{
	int a, x,y;
	double temp[ydim][xdim];
	
	for(a=0; a<3; a++)
	{
		for(y=0; y<ydim; y++)
			for(x=0; x<xdim; x++)
			{
				double A = in[y][x];
				double lt = x<=0 ? 0 : in[y][x-1];
				double up = y<=0 ? 0 : in[y-1][x];
				double rt = x>=xdim-1 ? 0 : in[y][x+1];
				double dn = y>=ydim-1 ? 0 : in[y+1][x];
				temp[y][x] = 
					(A + lt/3 + up/3 + rt/3 + dn/2 + (rand()*0.07/RAND_MAX)) / 2.57;
			}
		for(y=0; y<ydim; y++)
			for(x=0; x<xdim; x++)
				in[y][x] = temp[y][x];
	}
	
	if(!Draw)return;
	
	for(y=0; y<ydim; y++)
	{
		printf("\33[%d;%dH", y+1, 80-xdim+1);
		for(x=0; x<xdim; x++)
		{
			int R=in[y][x]*(rmax-1);
			int G=in[y][x]*(gmax-1)/2;
			int B=in[y][x]*(bmax-1)/4;
			
			SetAttr(Lookup[R][G][B]>>8);
			FlushSetAttr();
			putchar(Lookup[R][G][B]&255);
		}
	}
	SetAttr(7);
	FlushSetAttr();
	fflush(stdout);
}

static void Train(void)
{
	int x,y, a;
	int xofs = rand()%(xdim/3);
	int yofs = rand()%3;
	
	double amp = 0.7 + rand()*0.3 / RAND_MAX;
	
	for(y=0; y<ydim; y++)
		for(x=0; x<xdim; x++)
			in[y][x] = dah[((y+ydim-yofs)%ydim)*xdim 
			              + (x+xdim-xofs)%xdim]-'.' ? amp : 0;
			
	Noisify(1);
	
	for(x=0; x<numout; x++)
		out[x] = x ? 0.1 : 0.9;
			
	BrainTrain((double *)&in, out, 0.01, 0.2); /* err, rate */
	
	for(a=0; a<50; a++)
	{
		int spikecount = rand() % (1 + rand() % (1 + rand()%10));
		
		for(y=0; y<ydim; y++)
			for(x=0; x<xdim; x++)
			{
				int spike = rand()%(xdim*ydim);
				in[y][x] = spike<spikecount ? 3*amp : 0;
			}
				
		Noisify(1);
		
		for(x=0; x<numout; x++)out[x] = 0.1;
				
		BrainTrain((double *)&in, out, 0.02, 0.3); /* err, rate */
	}
}

static int Quitti=0;

static void Blah(int i)
{
	i=i;
	Quitti=1;
}

int main(void)
{
	int i;
	
	srand(time(NULL));
	
	puts("Loading brains");
	InitBrain(xdim*ydim, V5_HIDDENLAYERCOUNT, numout);
	puts("Initializing colors");
	MkTab();
	
	signal(SIGINT, Blah);
	
	printf("\33(U");
	
	for(i=1; !Quitti; i++)
	{
		printf("Training %d\n", i);
		Train();		
	}	
	
	DoneBrain();
	
	return 0;
}
